from typing import Any, Dict, Optional

import pytest
from requests_mock import Mocker

from philipstv import PhilipsTV, PhilipsTVError

HOST = "192.168.0.1"
PORT = 1926
PAYLOAD = {"<key>": "<value>"}

PATHS = [
    pytest.param("random/path", f"https://{HOST}:{PORT}/random/path", id="no leading slash"),
    pytest.param("/random/path", f"https://{HOST}:{PORT}/random/path", id="leading slash present"),
]
RESPONSES = [
    pytest.param({"<resp_key>": "<resp_value>"}, id="response present"),
    pytest.param(None, id="no response"),
]


def test_tv_auth() -> None:
    first_credentials = ("<first key>", "<first secret>")
    second_credentials = ("<second key>", "<second secret>")

    tv = PhilipsTV(HOST, PORT, first_credentials)
    assert tv.auth == first_credentials

    tv.auth = second_credentials
    assert tv.auth == second_credentials


@pytest.mark.parametrize("path, expected_url", PATHS)
@pytest.mark.parametrize("expected_response", RESPONSES)
def test_tv_post(
    path: str, expected_url: str, expected_response: Optional[Dict[str, Any]], requests_mock: Mocker
) -> None:
    requests_mock.post(expected_url, json=expected_response)

    tv = PhilipsTV(HOST, PORT)
    actual_response = tv.post(path, PAYLOAD)

    assert requests_mock.last_request
    assert requests_mock.last_request.json() == PAYLOAD
    assert actual_response == expected_response


@pytest.mark.parametrize("path, expected_url", PATHS)
@pytest.mark.parametrize("expected_response", RESPONSES)
def test_tv_get(
    path: str, expected_url: str, expected_response: Optional[Dict[str, Any]], requests_mock: Mocker
) -> None:
    requests_mock.get(expected_url, json=expected_response)

    tv = PhilipsTV(HOST, PORT)
    actual_response = tv.get(path)

    assert requests_mock.last_request
    assert actual_response == expected_response


@pytest.mark.parametrize("status_code", [401, 404, 500])
def test_tv_error(status_code: int, requests_mock: Mocker) -> None:
    path = "random/path"
    url = f"https://{HOST}:{PORT}/{path}"
    expected_message = f"POST request to {url} failed with status {status_code}"
    requests_mock.post(url, status_code=status_code)

    tv = PhilipsTV(HOST, PORT)

    with pytest.raises(PhilipsTVError, match=expected_message) as excinfo:
        tv.post(path)

    exception = excinfo.value
    assert exception.method == "POST"
    assert exception.url == url
    assert exception.status_code == status_code



from typing import Any, Dict, Optional, Set

from philipstv.exceptions import PhilipsTVError
from philipstv.tv import PhilipsTV
from philipstv.types import Credentials


class FakePhilipsTV(PhilipsTV):
    def __init__(
        self,
        post_responses: Optional[Dict[str, Any]] = None,
        get_responses: Optional[Dict[str, Any]] = None,
    ) -> None:
        self.host = ""
        self._auth: Optional[Credentials] = None
        self.post_requests: Dict[str, Any] = {}
        self.get_requests: Set[str] = set()

        self.post_responses = post_responses or {}
        self.get_responses = get_responses or {}

    @property
    def auth(self) -> Optional[Credentials]:
        return self._auth

    @auth.setter
    def auth(self, value: Optional[Credentials]) -> None:
        self._auth = value

    def post(self, path: str, payload: Any = None) -> Any:
        self.post_requests[path] = payload
        try:
            return self._raise_or_return(self.post_responses[path])
        except KeyError:
            raise PhilipsTVError("POST", path, 404)

    def get(self, path: str) -> Any:
        self.get_requests.add(path)
        try:
            return self._raise_or_return(self.get_responses[path])
        except KeyError:
            raise PhilipsTVError("GET", path, 404)

    @staticmethod
    def _raise_or_return(value: Any) -> Any:
        if isinstance(value, Exception):
            raise value
        return value



from typing import Any, Type

import pytest

from philipstv import (
    PhilipsTVAPI,
    PhilipsTVAPIMalformedResponseError,
    PhilipsTVAPIUnauthorizedError,
    PhilipsTVError,
)
from philipstv.model import (
    AllChannels,
    AmbilightColor,
    AmbilightLayer,
    AmbilightMode,
    AmbilightModeValue,
    AmbilightPower,
    AmbilightPowerValue,
    AmbilightTopology,
    Application,
    ApplicationComponent,
    ApplicationIntent,
    Applications,
    ApplicationShort,
    Channel,
    ChannelID,
    ChannelList,
    ChannelShort,
    CurrentChannel,
    CurrentVolume,
    DeviceInfo,
    InputKey,
    InputKeyValue,
    PairingAuthInfo,
    PairingGrantPayload,
    PairingRequestPayload,
    PairingRequestResponse,
    PairingResponse,
    PowerState,
    PowerStateValue,
    SetChannel,
    Volume,
)
from tests.fakes import FakePhilipsTV

DEVICE_INFO = DeviceInfo(
    id="<device_id>",
    device_name="<device_name>",
    device_os="<device_os>",
    app_id="<app_id>",
    app_name="<app_name>",
    type="<type>",
)


def test_host() -> None:
    expected_host = "192.168.0.66"
    fake_tv = FakePhilipsTV()
    fake_tv.host = expected_host

    result = PhilipsTVAPI(fake_tv).host

    assert result == expected_host


def test_auth() -> None:
    expected_auth = ("<id>", "<key>")
    fake_tv = FakePhilipsTV()
    api = PhilipsTVAPI(fake_tv)

    api.auth = expected_auth

    assert api.auth == expected_auth
    assert fake_tv.auth == expected_auth


def test_pair_request() -> None:
    fake_tv = FakePhilipsTV(
        post_responses={
            "6/pair/request": {
                "error_id": "SUCCESS",
                "error_text": "Authorization required",
                "auth_key": "<key>",
                "timestamp": 12345,
                "timeout": 60,
            }
        }
    )

    result = PhilipsTVAPI(fake_tv).pair_request(
        PairingRequestPayload(scope=["read", "write", "control"], device=DEVICE_INFO)
    )

    assert fake_tv.post_requests["6/pair/request"] == {
        "scope": ["read", "write", "control"],
        "device": DEVICE_INFO.dump(),
    }
    assert result == PairingRequestResponse(
        error_id="SUCCESS",
        error_text="Authorization required",
        auth_key="<key>",
        timestamp=12345,
        timeout=60,
    )


def test_pair_grant() -> None:
    fake_tv = FakePhilipsTV(
        post_responses={
            "6/pair/grant": {
                "error_id": "SUCCESS",
                "error_text": "Pairing completed",
            }
        }
    )

    result = PhilipsTVAPI(fake_tv).pair_grant(
        PairingGrantPayload(
            auth=PairingAuthInfo(pin="<pin>", auth_timestamp=12345, auth_signature="<signature>"),
            device=DEVICE_INFO,
        )
    )

    assert fake_tv.post_requests["6/pair/grant"] == {
        "auth": {
            "pin": "<pin>",
            "auth_timestamp": 12345,
            "auth_signature": "<signature>",
        },
        "device": DEVICE_INFO.dump(),
    }
    assert result == PairingResponse(
        error_id="SUCCESS",
        error_text="Pairing completed",
    )


def test_get_powerstate() -> None:
    fake_tv = FakePhilipsTV(get_responses={"6/powerstate": {"powerstate": "On"}})

    result = PhilipsTVAPI(fake_tv).get_powerstate()

    assert result == PowerState(powerstate=PowerStateValue.ON)


def test_set_powerstate() -> None:
    fake_tv = FakePhilipsTV(post_responses={"6/powerstate": None})

    PhilipsTVAPI(fake_tv).set_powerstate(PowerState(powerstate=PowerStateValue.STANDBY))

    assert fake_tv.post_requests == {"6/powerstate": {"powerstate": "Standby"}}


def test_get_volume() -> None:
    fake_tv = FakePhilipsTV(
        get_responses={"6/audio/volume": {"muted": False, "current": 15, "min": 0, "max": 60}}
    )

    result = PhilipsTVAPI(fake_tv).get_volume()

    assert result == CurrentVolume(current=15, muted=False, min=0, max=60)


def test_set_volume() -> None:
    fake_tv = FakePhilipsTV(post_responses={"6/audio/volume": None})

    PhilipsTVAPI(fake_tv).set_volume(Volume(current=10))

    assert fake_tv.post_requests == {"6/audio/volume": {"muted": False, "current": 10}}


def test_get_current_channel() -> None:
    fake_tv = FakePhilipsTV(
        get_responses={
            "6/activities/tv": {
                "channel": {"ccid": 35, "preset": "10", "name": "TVN HD"},
                "channelList": {"id": "list", "version": "7"},
            }
        }
    )

    result = PhilipsTVAPI(fake_tv).get_current_channel()

    assert result == CurrentChannel(
        channel=ChannelShort(ccid=35, preset="10", name="TVN HD"),
        channel_list=ChannelList(id="list", version="7"),
    )


def test_set_current_channel() -> None:
    fake_tv = FakePhilipsTV(post_responses={"6/activities/tv": None})

    PhilipsTVAPI(fake_tv).set_channel(SetChannel(channel=ChannelID(ccid=30)))

    assert fake_tv.post_requests == {
        "6/activities/tv": {"channel": {"ccid": 30}, "channelList": {"id": "allcab"}}
    }


def test_get_all_channels() -> None:
    fake_tv = FakePhilipsTV(
        get_responses={
            "6/channeldb/tv/channelLists/all": {
                "version": 1,
                "id": "all",
                "listType": "MixedSources",
                "medium": "mixed",
                "operator": "OPER",
                "installCountry": "Poland",
                "channel": [
                    {
                        "ccid": 35,
                        "preset": "1",
                        "name": "TVPiS 1 HD",
                        "onid": 1537,
                        "tsid": 24,
                        "sid": 2403,
                        "serviceType": "audio_video",
                        "type": "DVB_C",
                        "logoVersion": 33,
                    }
                ],
            },
        }
    )

    result = PhilipsTVAPI(fake_tv).get_all_channels()

    assert result == AllChannels(
        version=1,
        id="all",
        list_type="MixedSources",
        medium="mixed",
        operator="OPER",
        install_country="Poland",
        channel=[
            Channel(
                ccid=35,
                preset="1",
                name="TVPiS 1 HD",
                onid=1537,
                tsid=24,
                sid=2403,
                service_type="audio_video",
                type="DVB_C",
                logo_version=33,
            )
        ],
    )


def test_input_key() -> None:
    fake_tv = FakePhilipsTV(post_responses={"6/input/key": None})

    PhilipsTVAPI(fake_tv).input_key(InputKey(key=InputKeyValue.STANDBY))

    assert fake_tv.post_requests == {"6/input/key": {"key": "Standby"}}


def test_get_ambilight_power() -> None:
    fake_tv = FakePhilipsTV(get_responses={"6/ambilight/power": {"power": "On"}})

    result = PhilipsTVAPI(fake_tv).get_ambilight_power()

    assert result == AmbilightPower(power=AmbilightPowerValue.ON)


def test_set_ambilight_power() -> None:
    fake_tv = FakePhilipsTV(post_responses={"6/ambilight/power": None})

    PhilipsTVAPI(fake_tv).set_ambilight_power(AmbilightPower(power=AmbilightPowerValue.OFF))

    assert fake_tv.post_requests == {"6/ambilight/power": {"power": "Off"}}


def test_get_ambilight_topology() -> None:
    fake_tv = FakePhilipsTV(
        get_responses={
            "6/ambilight/topology": {"layers": 1, "left": 3, "top": 7, "right": 3, "bottom": 0}
        }
    )

    result = PhilipsTVAPI(fake_tv).get_ambilight_topology()

    assert result == AmbilightTopology(layers=1, left=3, top=7, right=3, bottom=0)


def test_get_abilight_mode() -> None:
    fake_tv = FakePhilipsTV(get_responses={"6/ambilight/mode": {"current": "internal"}})

    result = PhilipsTVAPI(fake_tv).get_ambilight_mode()

    assert result == AmbilightMode(current=AmbilightModeValue.INTERNAL)


def test_set_abilight_mode() -> None:
    fake_tv = FakePhilipsTV(post_responses={"6/ambilight/mode": None})

    PhilipsTVAPI(fake_tv).set_ambilight_mode(AmbilightMode(current=AmbilightModeValue.MANUAL))

    assert fake_tv.post_requests == {"6/ambilight/mode": {"current": "manual"}}


@pytest.mark.parametrize(
    "method, endpoint",
    [
        pytest.param("get_ambilight_measured", "6/ambilight/measured", id="measured"),
        pytest.param("get_ambilight_processed", "6/ambilight/processed", id="processed"),
        pytest.param("get_ambilight_cached", "6/ambilight/cached", id="cached"),
    ],
)
def test_get_ambilight_colors(method: str, endpoint: str) -> None:
    fake_tv = FakePhilipsTV(
        get_responses={
            endpoint: {
                "layer1": {
                    "left": {"0": {"r": 255, "g": 0, "b": 0}},
                    "top": {"0": {"r": 0, "g": 255, "b": 0}, "1": {"r": 0, "g": 0, "b": 0}},
                    "right": {"0": {"r": 0, "g": 0, "b": 255}},
                },
            }
        }
    )

    result = getattr(PhilipsTVAPI(fake_tv), method)()

    assert result["layer1"] == AmbilightLayer(
        left={"0": AmbilightColor(r=255, g=0, b=0)},
        top={"0": AmbilightColor(r=0, g=255, b=0), "1": AmbilightColor(r=0, g=0, b=0)},
        right={"0": AmbilightColor(r=0, g=0, b=255)},
    )


def test_set_ambilight_cached() -> None:
    fake_tv = FakePhilipsTV(post_responses={"6/ambilight/cached": None})

    PhilipsTVAPI(fake_tv).set_ambilight_cached(AmbilightColor(r=255, g=255, b=255))

    assert fake_tv.post_requests == {"6/ambilight/cached": {"r": 255, "g": 255, "b": 255}}


def test_get_applications() -> None:
    fake_tv = FakePhilipsTV(
        get_responses={
            "6/applications": {
                "version": 0,
                "applications": [
                    {
                        "intent": {
                            "component": {
                                "packageName": "org.droidtv.eum",
                                "className": "org.droidtv.eum.classname",
                            },
                            "action": "android.intent.action.MAIN",
                        },
                        "label": "Application",
                        "order": 0,
                        "id": "org.droidtv.eum.whatever",
                        "type": "app",
                    },
                ],
            }
        }
    )

    result = PhilipsTVAPI(fake_tv).get_applications()

    assert result == Applications(
        version=0,
        applications=[
            Application(
                intent=ApplicationIntent(
                    component=ApplicationComponent(
                        package_name="org.droidtv.eum",
                        class_name="org.droidtv.eum.classname",
                    ),
                    action="android.intent.action.MAIN",
                ),
                label="Application",
                order=0,
                id="org.droidtv.eum.whatever",
                type="app",
            )
        ],
    )


def test_launch_application() -> None:
    fake_tv = FakePhilipsTV(post_responses={"6/activities/launch": None})

    PhilipsTVAPI(fake_tv).launch_application(
        ApplicationShort(
            intent=ApplicationIntent(
                component=ApplicationComponent(
                    package_name="org.droidtv.eum",
                    class_name="org.droidtv.eum.classname",
                ),
                action="android.intent.action.MAIN",
            ),
        )
    )

    assert fake_tv.post_requests == {
        "6/activities/launch": {
            "intent": {
                "component": {
                    "packageName": "org.droidtv.eum",
                    "className": "org.droidtv.eum.classname",
                },
                "action": "android.intent.action.MAIN",
            },
        },
    }


@pytest.mark.parametrize(
    "response, expected_exception",
    [
        pytest.param(
            PhilipsTVError("GET", "6/powerstate", 401),
            PhilipsTVAPIUnauthorizedError,
            id="unauthorized request error",
        ),
        pytest.param(
            {"foo": "bar"},
            PhilipsTVAPIMalformedResponseError,
            id="malformed response error",
        ),
        pytest.param(
            PhilipsTVError("GET", "6/powerstate", 404),
            PhilipsTVError,
            id="unhandled error",
        ),
    ],
)
def test_api_error(response: Any, expected_exception: Type[Exception]) -> None:
    fake_tv = FakePhilipsTV(get_responses={"6/powerstate": response})

    with pytest.raises(expected_exception):
        PhilipsTVAPI(fake_tv).get_powerstate()



import json
import logging
from pathlib import Path
from typing import Optional

from appdirs import user_data_dir
from pydantic import BaseModel, ValidationError

_LOGGER = logging.getLogger(__name__)


DATA_FILE = Path(user_data_dir("philipstv", "cyran.dev")) / "data.json"


class HostData(BaseModel):
    host: str
    id: str
    key: str


class PhilipsTVData(BaseModel):
    last_host: HostData

    @classmethod
    def load(cls) -> Optional["PhilipsTVData"]:
        try:
            _LOGGER.debug("Trying to load application data from %s", DATA_FILE)
            return cls.model_validate(json.loads(DATA_FILE.read_text()))
        except FileNotFoundError:
            _LOGGER.debug("Data file not found")
            return None
        except ValidationError:
            return None
        finally:
            _LOGGER.debug("Application data loaded successfully")

    def save(self) -> None:
        _LOGGER.debug("Saving application data to %s", DATA_FILE)
        if not DATA_FILE.exists():
            _LOGGER.debug("Data file doesn't exist, creating")
            DATA_FILE.parent.mkdir(parents=True, exist_ok=True)
            DATA_FILE.touch()
        DATA_FILE.write_text(self.model_dump_json())
        _LOGGER.debug("Application data saved successfully")



import json
import time
from pathlib import Path
from typing import Callable, List, Optional, Sequence, Union
from unittest.mock import Mock, call, create_autospec

import pytest
from click.testing import CliRunner, Result
from pytest import MonkeyPatch

from philipstv import (
    AmbilightColor,
    InputKeyValue,
    PhilipsTVPairingError,
    PhilipsTVRemote,
    PhilipsTVRemoteError,
)
from philipstv._cli import cli
from philipstv.exceptions import PhilipsError, PhilipsTVAPIUnauthorizedError, PhilipsTVError
from philipstv.model import PairingResponse
from philipstv.types import Credentials


@pytest.fixture(autouse=True)
def no_saved_data(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr("philipstv._data.DATA_FILE", Path("/shurelythiscannotexist"))


@pytest.fixture
def data_file(monkeypatch: MonkeyPatch, tmp_path: Path) -> Path:
    data_file = tmp_path / "data.json"
    monkeypatch.setattr("philipstv._data.DATA_FILE", data_file)
    return data_file


@pytest.fixture
def remote(monkeypatch: MonkeyPatch) -> Mock:
    mock_remote = create_autospec(PhilipsTVRemote)
    mock_remote.new.return_value = mock_remote
    monkeypatch.setattr("philipstv._cli.PhilipsTVRemote", mock_remote)
    return mock_remote  # type: ignore


@pytest.fixture
def sleep_mock(monkeypatch: MonkeyPatch) -> Mock:
    mock_sleep = create_autospec(time.sleep)
    monkeypatch.setattr("philipstv._cli.time", Mock(sleep=mock_sleep))
    return mock_sleep  # type: ignore


def run(*args: str, input: Optional[str] = None) -> Result:
    return CliRunner().invoke(cli, args, input=input)


def run_with_auth(*args: str, input: Optional[str] = None) -> Result:
    return run("--host", "<host>", "--id", "<id>", "--key", "<key>", *args, input=input)


@pytest.mark.parametrize(
    "args, expected_error",
    [
        (
            ["--host", "192.168.0.1", "power"],
            "Options --host, --id and --key have to be used together.",
        ),
        (
            ["--id", "user", "power"],
            "Options --host, --id and --key have to be used together.",
        ),
        (
            ["--key", "69420", "power"],
            "Options --host, --id and --key have to be used together.",
        ),
        (
            ["--save", "power"],
            "--save requires giving --host, --id and --key.",
        ),
        (
            ["power"],
            "No TV data (--host, --id, --key) given or saved.",
        ),
        (
            ["pair"],
            "No host given (--host).",
        ),
        (
            ["--host", "192.168.0.1", "--id", "user", "--key", "secret", "pair"],
            "Option --key is invalid in pairing context.",
        ),
    ],
)
def test_errors(args: Sequence[str], expected_error: str) -> None:
    result = run(*args)

    assert result.exit_code != 0
    assert expected_error in result.stderr


def test_saves_data(data_file: Path, remote: Mock) -> None:
    given_host = "<host>"
    given_id = "<id>"
    given_key = "<key>"
    remote.get_power.return_value = True

    assert data_file.exists() is False

    run("--host", given_host, "--id", given_id, "--key", given_key, "--save", "power", "get")

    assert data_file.exists() is True
    assert json.loads(data_file.read_text()) == {
        "last_host": {"host": given_host, "id": given_id, "key": given_key}
    }


def test_reads_saved_data(data_file: Path, remote: Mock) -> None:
    given_host = "<host>"
    given_id = "<id>"
    given_key = "<key>"
    data_file.touch()
    data_file.write_text(
        json.dumps({"last_host": {"host": given_host, "id": given_id, "key": given_key}})
    )

    run("power", "get")

    remote.new.assert_called_once_with(given_host, (given_id, given_key))


def test_ignores_malformed_saved_data(data_file: Path, remote: Mock) -> None:
    data_file.touch()
    data_file.write_text(json.dumps({"whatever": {}}))

    result = run("power", "get")

    assert result.exit_code != 0
    assert "No TV data" in result.stderr


def test_pair(remote: Mock) -> None:
    given_id = "<myid>"
    given_pin = "<pin>"

    def mock_pair(callback: Callable[[], str], id: Optional[str] = None) -> Credentials:
        assert callback() == given_pin
        return (id or "<defaultid>", "<secret>")

    remote.pair.side_effect = mock_pair

    result = run("--host", "192.168.0.1", "--id", given_id, "pair", input=given_pin)

    assert f"Pairing successful!\nID:\t{given_id}\nKey:\t<secret>" in result.stdout


def test_pair_save(remote: Mock, data_file: Path) -> None:
    given_host = "<host>"
    given_id = "<id>"
    given_key = "<key>"
    remote.pair.return_value = (given_id, given_key)

    assert data_file.exists() is False

    run("--host", given_host, "--id", given_id, "--save", "pair")

    assert data_file.exists() is True
    assert json.loads(data_file.read_text()) == {
        "last_host": {"host": given_host, "id": given_id, "key": given_key}
    }


def test_pair_error(remote: Mock) -> None:
    remote.pair.side_effect = PhilipsTVPairingError(
        PairingResponse(error_id="ERR_ID", error_text="<error>")
    )

    result = run("--host", "192.168.0.1", "pair")

    assert result.exit_code != 0
    assert "ERR_ID <error>" in result.stderr


def test_power_get(remote: Mock) -> None:
    remote.get_power.return_value = False

    result = run_with_auth("power", "get")

    assert result.stdout == "off\n"


def test_power_set(remote: Mock) -> None:
    run_with_auth("power", "set", "on")

    remote.set_power.assert_called_once_with(True)


def test_volume_get(remote: Mock) -> None:
    remote.get_volume.return_value = 15

    result = run_with_auth("volume", "get")

    assert result.stdout == "15\n"


def test_volume_set(remote: Mock) -> None:
    run_with_auth("volume", "set", "25")

    remote.set_volume.assert_called_once_with(25)


def test_channel_get(remote: Mock) -> None:
    remote.get_current_channel.return_value = "TVN HD"

    result = run_with_auth("channel", "get")

    assert result.stdout == "TVN HD\n"


def test_channel_list(remote: Mock) -> None:
    remote.get_all_channels.return_value = {1: "TVP 1 HD", 3: "Polsat HD", 5: "TVN HD"}

    result = run_with_auth("channel", "list")

    assert result.stdout == "1\tTVP 1 HD\n3\tPolsat HD\n5\tTVN HD\n"


@pytest.mark.parametrize(
    "channel, expected",
    [
        ("TVN HD", "TVN HD"),
        ("17", 17),
    ],
)
def test_channel_set(remote: Mock, channel: str, expected: Union[str, int]) -> None:
    run_with_auth("channel", "set", channel)

    remote.set_channel.assert_called_once_with(expected)


def test_channel_set_error(remote: Mock) -> None:
    remote.set_channel.side_effect = PhilipsTVRemoteError("<error message>")

    result = run_with_auth("channel", "set", "whatever")

    assert result.exit_code != 0
    assert result.stderr == "Error: <error message>\n"


@pytest.mark.parametrize(
    "args, expected_keys, expected_sleep_calls",
    [
        ([], [], 0),
        (["home"], [InputKeyValue.HOME], 0),
        (["right", "ok"], [InputKeyValue.CURSOR_RIGHT, InputKeyValue.CONFIRM], 0),
        (["--delay", "100", "home"], [InputKeyValue.HOME], 0),
        (["--delay", "0", "ok", "ok"], [InputKeyValue.CONFIRM, InputKeyValue.CONFIRM], 0),
        (["--delay", "100", "ok", "ok"], [InputKeyValue.CONFIRM, InputKeyValue.CONFIRM], 1),
    ],
)
def test_key(
    remote: Mock,
    sleep_mock: Mock,
    args: List[str],
    expected_keys: List[InputKeyValue],
    expected_sleep_calls: int,
) -> None:
    run_with_auth("key", *args)

    assert remote.input_key.call_args_list == [call(key) for key in expected_keys]
    assert sleep_mock.call_count == expected_sleep_calls


def test_ambilight_power_get(remote: Mock) -> None:
    remote.get_ambilight_power.return_value = False

    result = run_with_auth("ambilight", "power", "get")

    assert result.stdout == "off\n"


def test_ambilight_power_set(remote: Mock) -> None:
    run_with_auth("ambilight", "power", "set", "on")

    remote.set_ambilight_power.assert_called_once_with(True)


def test_ambilight_color_set(remote: Mock) -> None:
    run_with_auth("ambilight", "color", "set", "255", "0", "0", "--top", "0", "255", "0")

    remote.set_ambilight_color.assert_called_once_with(
        color=AmbilightColor(r=255, g=0, b=0),
        left=None,
        top=AmbilightColor(r=0, g=255, b=0),
        right=None,
        bottom=None,
    )


def test_ambilight_color_set_error(remote: Mock) -> None:
    result = run_with_auth("ambilight", "color", "set")

    remote.set_ambilight_color.assert_not_called()

    assert result.exit_code != 0
    assert "Error" in result.stderr


def test_app_list(remote: Mock) -> None:
    remote.get_applications.return_value = ["Netflix", "YouTube", "TED"]

    result = run_with_auth("app", "list")

    assert result.stdout == "Netflix\nYouTube\nTED\n"


def test_app_launch(remote: Mock) -> None:
    run_with_auth("app", "launch", "YouTube")

    remote.launch_application.assert_called_once_with("YouTube")


def test_app_launch_error(remote: Mock) -> None:
    remote.launch_application.side_effect = PhilipsTVRemoteError("<error message>")

    result = run_with_auth("app", "launch", "whatever")

    assert result.exit_code != 0
    assert result.stderr == "Error: <error message>\n"


@pytest.mark.parametrize(
    "exception, expected_message",
    [
        pytest.param(
            PhilipsTVRemoteError("<error>"),
            "Error: <error>",
            id="remote usage error",
        ),
        pytest.param(
            PhilipsTVAPIUnauthorizedError("<method>", "<path>"),
            "Error: Could not authenticate with the TV.",
            id="authentication error",
        ),
        pytest.param(
            PhilipsTVError("<method>", "<url>"),
            "Error: Could not connect with the TV.",
            id="connection error",
        ),
        pytest.param(
            PhilipsError("<error>"),
            "Error: Unknown error occured.",
            id="unknown error",
        ),
    ],
)
def test_handle_exceptions(remote: Mock, exception: Exception, expected_message: str) -> None:
    remote.get_power.side_effect = exception

    result = run_with_auth("power", "get")

    assert result.exit_code != 0
    assert expected_message in result.stderr



from unittest.mock import create_autospec

import pytest

from philipstv import DeviceInfo, PhilipsTVAPI, PhilipsTVPairer, PhilipsTVPairingError
from philipstv._utils import create_signature
from philipstv.model import (
    PairingAuthInfo,
    PairingGrantPayload,
    PairingRequestPayload,
    PairingRequestResponse,
    PairingResponse,
)
from philipstv.pairing import _SECRET

DEVICE_INFO = DeviceInfo(
    id="<device_id>",
    device_name="<device_name>",
    device_os="<device_os>",
    app_id="<app_id>",
    app_name="<app_name>",
    type="<type>",
)
SIGNATURE = create_signature(_SECRET, "12345<pin>".encode()).decode()
REQUEST_REPONSE_OK = PairingRequestResponse(
    error_id="SUCCESS",
    error_text="Authorization required",
    auth_key="<key>",
    timestamp=12345,
    timeout=60,
)
GRANT_RESPONSE_OK = PairingResponse(
    error_id="SUCCESS",
    error_text="Pairing completed",
)
REQUEST_REPONSE_ERR = PairingResponse(
    error_id="CONCURRENT_PAIRING",
    error_text="Another pairing is in process",
)
GRANT_RESPONSE_ERR = PairingResponse(
    error_id="INVALID_PIN",
    error_text="Invalid authentication parameters",
)


def test_pair_happy_path() -> None:
    api_mock = create_autospec(PhilipsTVAPI)
    api_mock.pair_request.return_value = REQUEST_REPONSE_OK
    api_mock.pair_grant.return_value = GRANT_RESPONSE_OK
    expected_credentials = ("<device_id>", "<key>")

    credentials = PhilipsTVPairer(api_mock, DEVICE_INFO).pair(lambda: "<pin>")

    api_mock.pair_request.assert_called_once_with(
        PairingRequestPayload(scope=["read", "write", "control"], device=DEVICE_INFO)
    )
    api_mock.pair_grant.assert_called_once_with(
        PairingGrantPayload(
            auth=PairingAuthInfo(pin="<pin>", auth_timestamp=12345, auth_signature=SIGNATURE),
            device=DEVICE_INFO,
        )
    )
    assert credentials == expected_credentials
    assert api_mock.auth == expected_credentials


@pytest.mark.parametrize(
    "pair_response, grant_response, expected_message",
    [
        (REQUEST_REPONSE_ERR, GRANT_RESPONSE_OK, ".*CONCURRENT_PAIRING.*"),
        (REQUEST_REPONSE_OK, GRANT_RESPONSE_ERR, ".*INVALID_PIN.*"),
    ],
)
def test_pair_error(
    pair_response: PairingResponse, grant_response: PairingResponse, expected_message: str
) -> None:
    api_mock = create_autospec(PhilipsTVAPI)
    api_mock.pair_request.return_value = pair_response
    api_mock.pair_grant.return_value = grant_response
    pairer = PhilipsTVPairer(api_mock, DEVICE_INFO)

    with pytest.raises(PhilipsTVPairingError, match=expected_message):
        pairer.pair(lambda: "<pin>")

    assert api_mock.auth is None



from typing import Union
from unittest.mock import Mock, create_autospec

import pytest
from pytest import MonkeyPatch

from philipstv import PhilipsTVAPI, PhilipsTVPairer, PhilipsTVRemote, PhilipsTVRemoteError
from philipstv.model import (
    AllChannels,
    AmbilightColor,
    AmbilightColors,
    AmbilightLayer,
    AmbilightPower,
    AmbilightPowerValue,
    AmbilightTopology,
    Application,
    ApplicationComponent,
    ApplicationIntent,
    Applications,
    Channel,
    ChannelID,
    ChannelList,
    ChannelShort,
    CurrentChannel,
    CurrentVolume,
    DeviceInfo,
    InputKey,
    InputKeyValue,
    PowerState,
    PowerStateValue,
    SetChannel,
    Volume,
)

CHANNELS = AllChannels(
    version=1,
    id="all",
    list_type="MixedSources",
    medium="mixed",
    operator="OPER",
    install_country="Poland",
    channel=[
        Channel(
            ccid=35,
            preset="1",
            name="Polsat HD",
            onid=1537,
            tsid=24,
            sid=2403,
            service_type="audio_video",
            type="DVB_C",
            logo_version=33,
        ),
        Channel(
            ccid=40,
            preset="3",
            name="TVN HD",
            onid=666,
            tsid=24,
            sid=2403,
            service_type="audio_video",
            type="DVB_C",
            logo_version=33,
        ),
    ],
)

APPLICATION_SPOTIFY = Application(
    intent=ApplicationIntent(
        component=ApplicationComponent(
            package_name="com.spotify.tv.android",
            class_name="com.spotify.tv.android.SpotifyTVActivity",
        ),
        action="android.intent.action.MAIN",
    ),
    label="Spotify",
    order=0,
    id="com.spotify.tv.android.SpotifyTVActivity-com.spotify.tv.android",
    type="app",
)
APPLICATION_NETFLIX = Application(
    intent=ApplicationIntent(
        component=ApplicationComponent(
            package_name="com.netflix.ninja",
            class_name="com.netflix.ninja.MainActivity",
        ),
        action="android.intent.action.MAIN",
    ),
    label="Netflix",
    order=0,
    id="com.netflix.ninja.MainActivity-com.netflix.ninja",
    type="app",
)
APPLICATIONS = Applications(
    version=0,
    applications=[APPLICATION_SPOTIFY, APPLICATION_NETFLIX],
)


@pytest.fixture
def api_mock() -> Mock:
    return create_autospec(PhilipsTVAPI, spec_set=True, instance=True)  # type: ignore


def test_host(api_mock: Mock) -> None:
    expected_host = "192.168.0.66"
    api_mock.host = expected_host

    result = PhilipsTVRemote(api_mock).host

    assert result == expected_host


def test_auth(api_mock: PhilipsTVAPI) -> None:
    expected_credentials = ("<key>", "<secret>")
    remote = PhilipsTVRemote(api_mock)

    remote.auth = expected_credentials

    assert remote.auth == expected_credentials
    assert api_mock.auth == expected_credentials


def test_pair(api_mock: Mock, monkeypatch: MonkeyPatch) -> None:
    given_id = "<id>"
    pairer_mock = create_autospec(PhilipsTVPairer)
    pairer_mock.return_value = pairer_mock
    monkeypatch.setattr("philipstv.remote.PhilipsTVPairer", pairer_mock)

    def fake_callback() -> str:
        return "str"

    PhilipsTVRemote(api_mock).pair(fake_callback, given_id)

    pairer_mock.pair.assert_called_once_with(fake_callback)
    device_info = pairer_mock.call_args.args[1]
    assert isinstance(device_info, DeviceInfo)
    assert device_info.id == given_id


def test_pair_no_id(api_mock: Mock, monkeypatch: MonkeyPatch) -> None:
    pairer_mock = create_autospec(PhilipsTVPairer)
    pairer_mock.return_value = pairer_mock
    monkeypatch.setattr("philipstv.remote.PhilipsTVPairer", pairer_mock)

    PhilipsTVRemote(api_mock).pair(lambda: "str")

    device_info = pairer_mock.call_args.args[1]
    assert isinstance(device_info, DeviceInfo)
    assert device_info.id.isalnum()
    assert len(device_info.id) == 16


def test_get_power(api_mock: Mock) -> None:
    api_mock.get_powerstate.return_value = PowerState(powerstate=PowerStateValue.STANDBY)

    result = PhilipsTVRemote(api_mock).get_power()

    assert result is False


def test_set_power(api_mock: Mock) -> None:
    PhilipsTVRemote(api_mock).set_power(True)

    api_mock.set_powerstate.assert_called_once_with(PowerState(powerstate=PowerStateValue.ON))


def test_get_volume(api_mock: Mock) -> None:
    api_mock.get_volume.return_value = CurrentVolume(muted=False, current=15, min=0, max=60)

    result = PhilipsTVRemote(api_mock).get_volume()

    assert result == 15


def test_set_volume(api_mock: Mock) -> None:
    PhilipsTVRemote(api_mock).set_volume(20)

    api_mock.set_volume.assert_called_once_with(Volume(current=20, muted=False))


def test_get_current_channel(api_mock: Mock) -> None:
    api_mock.get_current_channel.return_value = CurrentChannel(
        channel=ChannelShort(ccid=5, preset="10", name="TVN HD"),
        channel_list=ChannelList(id="allcab", version="1"),
    )

    result = PhilipsTVRemote(api_mock).get_current_channel()

    assert result == "TVN HD"


@pytest.mark.parametrize(
    "input, expected",
    [
        (1, SetChannel(channel=ChannelID(ccid=35))),
        ("Polsat HD", SetChannel(channel=ChannelID(ccid=35))),
        (3, SetChannel(channel=ChannelID(ccid=40))),
        ("TVN HD", SetChannel(channel=ChannelID(ccid=40))),
    ],
)
def test_set_channel(api_mock: Mock, input: Union[int, str], expected: SetChannel) -> None:
    api_mock.get_all_channels.return_value = CHANNELS
    remote = PhilipsTVRemote(api_mock)

    remote.set_channel(input)
    api_mock.set_channel.assert_called_once_with(expected)

    remote.set_channel(input)
    api_mock.get_all_channels.assert_called_once()


def test_set_channel_error(api_mock: Mock) -> None:
    api_mock.get_current_channel.return_value = CHANNELS

    with pytest.raises(PhilipsTVRemoteError):
        PhilipsTVRemote(api_mock).set_channel("random channel")


def test_get_all_channels(api_mock: Mock) -> None:
    api_mock.get_all_channels.return_value = CHANNELS
    result = PhilipsTVRemote(api_mock).get_all_channels()

    assert result == {1: "Polsat HD", 3: "TVN HD"}


def test_input_key(api_mock: Mock) -> None:
    PhilipsTVRemote(api_mock).input_key(InputKeyValue.STANDBY)

    api_mock.input_key.assert_called_once_with(InputKey(key=InputKeyValue.STANDBY))


def test_get_ambilight_power(api_mock: Mock) -> None:
    api_mock.get_ambilight_power.return_value = AmbilightPower(power=AmbilightPowerValue.OFF)

    result = PhilipsTVRemote(api_mock).get_ambilight_power()

    assert result is False


def test_set_ambilight_power(api_mock: Mock) -> None:
    PhilipsTVRemote(api_mock).set_ambilight_power(True)

    api_mock.set_ambilight_power.assert_called_once_with(
        AmbilightPower(power=AmbilightPowerValue.ON)
    )


def test_set_ambilight_color(api_mock: Mock) -> None:
    PhilipsTVRemote(api_mock).set_ambilight_color(AmbilightColor(r=0, g=69, b=255))

    api_mock.set_ambilight_cached.assert_called_once_with(AmbilightColor(r=0, g=69, b=255))


def test_set_ambilight_color_sides(api_mock: Mock) -> None:
    left_color = AmbilightColor(r=255, g=0, b=0)
    top_color = AmbilightColor(r=0, g=255, b=0)
    right_color = AmbilightColor(r=0, g=0, b=255)
    bottom_color = AmbilightColor(r=125, g=0, b=125)
    topology = AmbilightTopology(layers=1, left=2, top=3, right=2, bottom=3)
    api_mock.get_ambilight_topology.return_value = topology

    PhilipsTVRemote(api_mock).set_ambilight_color(
        left=left_color, top=top_color, right=right_color, bottom=bottom_color
    )

    api_mock.set_ambilight_cached.assert_called_once_with(
        AmbilightColors(
            {
                "layer1": AmbilightLayer(
                    left={str(point): left_color for point in range(topology.left)},
                    top={str(point): top_color for point in range(topology.top)},
                    right={str(point): right_color for point in range(topology.right)},
                    bottom={str(point): bottom_color for point in range(topology.bottom)},
                )
            }
        )
    )


def test_get_applications(api_mock: Mock) -> None:
    api_mock.get_applications.return_value = APPLICATIONS

    result = PhilipsTVRemote(api_mock).get_applications()

    assert result == ["Spotify", "Netflix"]


@pytest.mark.parametrize(
    "app, expected",
    [
        ("Spotify", APPLICATION_SPOTIFY),
        ("Netflix", APPLICATION_NETFLIX),
    ],
)
def test_launch_application(api_mock: Mock, app: str, expected: ApplicationIntent) -> None:
    api_mock.get_applications.return_value = APPLICATIONS
    remote = PhilipsTVRemote(api_mock)

    remote.launch_application(app)
    api_mock.launch_application.assert_called_once_with(expected)

    remote.launch_application(app)
    api_mock.get_applications.assert_called_once()


def test_launch_application_error(api_mock: Mock) -> None:
    api_mock.get_applications.return_value = APPLICATIONS

    with pytest.raises(PhilipsTVRemoteError):
        PhilipsTVRemote(api_mock).launch_application("whatever")
